@@ -46,6 +46,7 @@
 #include "ipod-device.h"
 #include "glib-compat.h"
 #include "ipod-model-table.h"
+#include "ipod-prepare-sysinfo.h"
 
 #define g_free_if_not_null(o) \
     if(o != NULL) {           \
@@ -781,7 +782,12 @@ ipod_device_get_extended_sysinfo_path(IpodDevice *device)
     char *path;
     char *udi;
 
-    path = g_strdup_printf ("%sDevice/SysInfoExtended", device->priv->control_path);
+    if(!ipod_prepare_sysinfo (device->priv->serial_number.serial, 
+        device->priv->can_write, device->priv->mount_point, &path)) {
+        return NULL;
+    }
+
+    //g_debug("SysInfo PLIST XML: %s", path);
 
     if (g_file_test (path, G_FILE_TEST_EXISTS)) {
         return path;
@@ -974,9 +980,10 @@ ipod_device_read_sysinfo_hal(IpodDevice *device)
     xmlNodePtr root;
     xmlXPathContextPtr xpath;
 
-    device->priv->serial_number.serial = libhal_device_get_property_string(device->priv->hal_context,
-									   device->priv->hal_volume_id,
-									   "info.ipod.serial_number", NULL);
+    device->priv->serial_number.serial = 
+        g_strstrip (libhal_device_get_property_string (device->priv->hal_context,
+            device->priv->hal_volume_id,
+            "info.ipod.serial_number", NULL));
     
     path = ipod_device_get_extended_sysinfo_path (device);
     if (path == NULL) {
@@ -998,7 +1005,8 @@ ipod_device_read_sysinfo_hal(IpodDevice *device)
 
     xpath = xmlXPathNewContext(doc);
 
-    device->priv->serial_number.serial = ipod_device_get_dict_value (xpath, "/plist/dict", "SerialNumber");
+    device->priv->serial_number.serial = g_strstrip (ipod_device_get_dict_value (
+        xpath, "/plist/dict", "SerialNumber"));
 
     ipod_device_read_artwork_info (device, xpath);
 
@@ -1076,7 +1084,7 @@ ipod_device_read_sysinfo(IpodDevice *device)
         }
 
         if (strcmp (key, "pszSerialNumber") == 0) {
-            device->priv->serial_number.serial = g_strdup (value);
+            device->priv->serial_number.serial = g_strstrip (g_strdup (value));
         } else if (strcmp (key, "ModelNumStr") == 0) {
             device->priv->model_number = g_strdup (value);
         } else if (strcmp (key, "buildID") == 0) {
@@ -1139,11 +1147,23 @@ ipod_device_detect_writeable(IpodDevice *device)
     
     if(!libhal_device_property_exists(device->priv->hal_context, device->priv->hal_volume_id, 
         "volume.is_mounted_read_only", NULL)) {
-        device->priv->can_write = !libhal_device_get_property_bool(device->priv->hal_context, 
-            device->priv->hal_volume_id, "volume.is_mounted_read_only", NULL);
         return device->priv->can_write;
     }
 
+    if(libhal_device_property_exists(device->priv->hal_context, 
+        device->priv->hal_volume_id, "volume.fstype", NULL)) {
+        gchar *fstype = libhal_device_get_property_string(
+            device->priv->hal_context, device->priv->hal_volume_id, 
+            "volume.fstype", NULL);
+
+        if(strcmp(fstype, "hfsplus") == 0) {
+            g_free(fstype);
+            return FALSE;
+        }
+
+        g_free(fstype);
+    }
+
     itunes_dir = g_strdup_printf("%siTunes", device->priv->control_path);
     
     if(g_mkdir_with_parents(itunes_dir, 0755) == -1) {
